import * as Cesium from "cesium";
import {
    CallbackProperty,
    type Cartesian3,
    ConstantPositionProperty,
    ScreenSpaceEventHandler,
} from "cesium";

export default class DynamicDrawClass {
    private readonly viewer: Cesium.Viewer;
    private entities: Map<string, Cesium.Entity>;
    private handler: Cesium.ScreenSpaceEventHandler | null = null;
    private readonly ellipsoid: Cesium.Ellipsoid;

    constructor(viewer: Cesium.Viewer) {
        this.viewer = viewer;
        this.entities = new Map();
        this.ellipsoid = viewer.scene.globe.ellipsoid;
    }

    private removeHandler() {
        if (this.handler) {
            this.handler.destroy();
            this.handler = null;
        }
    }

    private pickPosition(event: Cesium.ScreenSpaceEventHandler.MotionEvent | Cesium.ScreenSpaceEventHandler.PositionedEvent): Cartesian3 | undefined {
        let position: Cesium.Cartesian3 | undefined;
        if ("endPosition" in event) {
            position = this.viewer.scene.camera.pickEllipsoid(event.endPosition, this.ellipsoid);
        } else if ("position" in event) {
            position = this.viewer.scene.camera.pickEllipsoid(event.position, this.ellipsoid);
        }
        return position;
    }

    private addEntity(entityOptions: Cesium.Entity.ConstructorOptions): Cesium.Entity {
        return this.viewer.entities.add(entityOptions);
    }

    private removeEntity(entity: Cesium.Entity): boolean {
        return this.viewer.entities.remove(entity);
    }

    /**
     * 动态点绘制
     */
    drawDynamicPoint(): Cesium.Entity | undefined {
        this.removeHandler();
        let activePosition: Cartesian3 | undefined;
        let dynamicPoint: Cesium.Entity | undefined;

        this.handler = new ScreenSpaceEventHandler(this.viewer.scene.canvas);

        this.handler.setInputAction((event:ScreenSpaceEventHandler.PositionedEvent) => {
            const position = this.pickPosition(event);
            if (position) {
                activePosition = position;
                if (!dynamicPoint) {
                    dynamicPoint = this.addEntity({
                        position: new CallbackProperty(() => activePosition, false),
                        point: {
                            color: Cesium.Color.RED,
                            pixelSize: 10,
                            outlineColor: Cesium.Color.GREEN,
                            outlineWidth: 1.5,
                            heightReference: Cesium.HeightReference.CLAMP_TO_GROUND,
                        },
                    });
                }
            }
        }, Cesium.ScreenSpaceEventType.MOUSE_MOVE);

        this.handler.setInputAction((event:ScreenSpaceEventHandler.PositionedEvent) => {
            const position = this.pickPosition(event);
            if (position && dynamicPoint) {
                activePosition = position;
                dynamicPoint.position = new ConstantPositionProperty(activePosition);
                this.removeHandler();
            }
        }, Cesium.ScreenSpaceEventType.LEFT_CLICK);

        return dynamicPoint;
    }

    /**
     * 动态直线或多边形绘制通用方法
     */
    private drawDynamicShape(isPolygon: boolean=false,isLine:boolean=false,isDynamicLien:boolean=false): Cesium.Entity | undefined {
        this.removeHandler();
        let activePositions: Cartesian3[] = [];
        let dynamicShape: Cesium.Entity | undefined;
        let edgeEntity: Cesium.Entity | undefined;
        const hierarchy = isPolygon ? new Cesium.PolygonHierarchy() : undefined;

        this.handler = new ScreenSpaceEventHandler(this.viewer.scene.canvas);

        this.handler.setInputAction((event:ScreenSpaceEventHandler.MotionEvent) => {
            const position = this.pickPosition(event);
            if (position) {
                if (activePositions.length > 0) {
                    activePositions[activePositions.length - 1] = position;
                    if (hierarchy) hierarchy.positions[activePositions.length - 1] = position;
                } else {
                    activePositions.push(position);
                    if (hierarchy) hierarchy.positions.push(position);
                }
                if (!edgeEntity) {
                    edgeEntity = this.addEntity({
                        polyline: {
                            positions: new CallbackProperty(() => activePositions, false),
                            width: 1,
                            material: Cesium.Color.RED.withAlpha(0.5),
                        },
                    });
                }
            }
        }, Cesium.ScreenSpaceEventType.MOUSE_MOVE);

        this.handler.setInputAction((event:ScreenSpaceEventHandler.PositionedEvent) => {
            const position = this.pickPosition(event);
            if (position) {
                activePositions.push(position);
                if (hierarchy) hierarchy.positions.push(position);
                if (!dynamicShape) {
                    dynamicShape = this.addEntity(
                        isPolygon ? {
                                polygon: {
                                    hierarchy: new CallbackProperty(() => hierarchy, false),
                                    material: Cesium.Color.WHITESMOKE.withAlpha(0.5),
                                    outlineWidth: 2,
                                    outlineColor:Cesium.Color.RED
                                },
                            }
                            : {
                                polyline: {
                                    positions: new CallbackProperty(() => activePositions, false),
                                    width: 3,
                                    material: Cesium.Color.BLUE.withAlpha(0.5),
                                },
                            }
                    );
                }
            }
        }, Cesium.ScreenSpaceEventType.LEFT_CLICK);

        this.handler.setInputAction(() => {
            if (activePositions.length > 1) {
                activePositions.pop();
                if (hierarchy) hierarchy.positions.pop();
            }
        }, Cesium.ScreenSpaceEventType.RIGHT_CLICK);

        this.handler.setInputAction(() => {
            this.removeHandler();
            if (isPolygon) activePositions.push(activePositions[0]);
            if (edgeEntity) this.removeEntity(edgeEntity);
        }, Cesium.ScreenSpaceEventType.LEFT_DOUBLE_CLICK);

        return dynamicShape;
    }

    drawDynamicLine(): Cesium.Entity | undefined {
        return this.drawDynamicShape(false);
    }

    drawDynamicPolygon(): Cesium.Entity | undefined {
        return this.drawDynamicShape(true);
    }


    /**
     * 移除图形
     */
    remove(id: string): boolean {
        const entity = this.entities.get(id);
        if (entity) {
            this.removeEntity(entity);
            this.entities.delete(id);
            return true;
        }
        return false;
    }

    /**
     * 清除所有绘制的图形
     */
    clearAll(): void {
        this.viewer.entities.removeAll();
        this.entities.clear();
    }
}
添加绘制线段